;;--------------------------------------------------------------------------------
;; $Header: /home/amb/pic/common/RCS/delay_ms.inc,v 1.6 2007/05/07 18:31:40 amb Exp $
;;
;; Delay for a number of milliseconds.
;;
;; Written by Andrew M. Bishop
;;
;; This file Copyright 2006 Andrew M. Bishop
;; It may be distributed under the GNU Public License, version 2, or
;; any higher version.  See section COPYING of the GNU Public license
;; for conditions under which this file may be redistributed.
;;
;;--------------------------------------------------------------------------------

;;--------------------------------------------------------------------------------
;;
;; Required Variables / Constants for milliscond delay function
;;
;; Constants:   CLOCK           - The PIC clock speed in Hz
;;
;; Variables:   delay_temp      - Stores temporary data
;;
;;--------------------------------------------------------------------------------


;;--------------------------------------------------------------------------------
;;
;; Subroutine to delay for a number of milliseconds (different versions for different clocks)
;;
;; Parameters:  w (in)          - Number of milliseconds to wait for
;;
;; RAM Bank:    any (in), unchanged (out)
;;
;;--------------------------------------------------------------------------------


        IF      CLOCK==1000000

;; Timing analysis (at 1 MHz clock)
;; Load and call       =  3                     =    3
;; Static overhead = A = 11 + 4 * 59            =  247
;; Outer loop      = B = 10 * (delay-1)         = - 10 +  10 * delay
;; Inner loop      = C =  4 * 60 * (delay-1)    = -240 + 240 * delay
;; Total           = 250 * delay

delay_ms
        goto    delayms#v(1)             ; 2 ins (A)
delayms#v(1)   movwf   delay_temp      ; 1 ins (A)
        movlw   60              ; 1 ins (A)

delay_ms_loop
        addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfsc   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        return                  ; 2 ins (A)

        nop                     ; 1 ins (B)
        movlw   61              ; 1 ins (B)
        goto    delay_ms_loop   ; 2 ins (B)


        ENDIF   ; CLOCK==1000000

;;--------------------------------------------------------------------------------

        IF      CLOCK==2000000

;; Timing analysis (at 2 MHz clock)
;; Load and call       =  3                     =    3
;; Static overhead = A =  9 + 4 * 122           =  497
;; Outer loop      = B = 12 * (delay-1)         = - 12 +  12 * delay
;; Inner loop      = C =  4 * 122 * (delay-1)   = -488 + 488 * delay
;; Total           = 500 * delay

delay_ms
        movwf   delay_temp      ; 1 ins (A)
        movlw   123             ; 1 ins (A)

delay_ms_loop
        addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfsc   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        return                  ; 2 ins (A)

        goto    delayms#v(2)             ; 2 ins (B)
delayms#v(2)    nop                     ; 1 ins (B)
        movlw   123             ; 1 ins (B)
        goto    delay_ms_loop   ; 2 ins (B)


        ENDIF   ; CLOCK==2000000

;;--------------------------------------------------------------------------------

        IF      CLOCK==4000000

;; Timing analysis (at 4 MHz clock)
;; Load and call       =  3                     =    3
;; Static overhead = A =  9 + 4 * 247           =  997
;; Outer loop      = B = 12 * (delay-1)         = - 12 +  12 * delay
;; Inner loop      = C =  4 * 247 * (delay-1)   = -988 + 988 * delay
;; Total           = 1000 * delay

delay_ms
        movwf   delay_temp      ; 1 ins (A)
        movlw   .248             ; 1 ins (A)

delay_ms_loop
        addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfsc   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        return                  ; 2 ins (A)

        goto    delayms#v(3)             ; 2 ins (B)
delayms#v(3)    nop                     ; 1 ins (B)
        movlw   .248             ; 1 ins (B)
        goto    delay_ms_loop   ; 2 ins (B)


        ENDIF   ; CLOCK==4000000

;;--------------------------------------------------------------------------------

        IF      CLOCK==8000000

;; Timing analysis (at 8 MHz clock)
;; Load and call       =  3                     =     3
;; Static overhead = A = 13 + 8 * 248           =  1997
;; Outer loop      = B = 16 * (delay-1)         = -  16 +   16 * delay
;; Inner loop      = C =  8 * 248 * (delay-1)   = -1984 + 1984 * delay
;; Total           = 2000 * delay

delay_ms
        movwf   delay_temp      ; 1 ins (A)
        movlw   .249             ; 1 ins (A)

delay_ms_loop
        goto    delayms#v(4)             ; 2 ins (A,B,C)
delayms#v(4)   goto    delayms#v(5)             ; 2 ins (A,B,C)
delayms#v(5)  addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfsc   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        return                  ; 2 ins (A)

        goto    delayms#v(6)             ; 2 ins (B)
delayms#v(6)    nop                     ; 1 ins (B)
        movlw   .249             ; 1 ins (B)
        goto    delay_ms_loop   ; 2 ins (B)


        ENDIF   ; CLOCK==8000000

;;--------------------------------------------------------------------------------

        IF      CLOCK==16000000

;; Timing analysis (at 16 MHz clock)
;; Load and call       =  3                     =     3
;; Static overhead = A = 13 + 16 * 249          =  3997
;; Outer loop      = B = 16 * (delay-1)         = -  16 +   16 * delay
;; Inner loop      = C = 16 * 249 * (delay-1)   = -3984 + 3984 * delay
;; Total           = 4000 * delay

delay_ms
        movwf   delay_temp      ; 1 ins (A)
        movlw   .250             ; 1 ins (A)
        goto    delay_ms_entry_A; 2 ins (A)

delay_ms_loop
        goto    delayms#v(7)             ; 2 ins (C)
delayms#v(7)     goto    delayms#v(8)             ; 2 ins (C)
delayms#v(8)    goto    delayms#v(9)             ; 2 ins (C)

delayms#v(9)
delay_ms_entry_B
        goto    delayms#v(10)             ; 2 ins (B,C)
delayms#v(10)   goto    delayms#v(11)             ; 2 ins (B,C)
delayms#v(11)
delay_ms_entry_A
        goto    delayms#v(12)             ; 2 ins (A,B,C)
delayms#v(12)  addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfsc   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        return                  ; 2 ins (A)

        nop                     ; 1 ins (B)
        movlw   .250             ; 1 ins (B)
        goto    delay_ms_entry_B; 2 ins (B)


        ENDIF   ; CLOCK==16000000

;;--------------------------------------------------------------------------------

        IF      CLOCK==20000000

;; Timing analysis (at 20 MHz clock)
;; Load and call       =  3                     =     3
;; Static overhead = A = 17 + 20 * 249          =  4997
;; Outer loop      = B = 20 * (delay-1)         = -  20 +   20 * delay
;; Inner loop      = C = 20 * 249 * (delay-1)   = -4980 + 4980 * delay
;; Total           = 5000 * delay

delay_ms
        movwf   delay_temp      ; 1 ins (A)
        movlw   .250             ; 1 ins (A)
        goto    delay_ms_entry_A; 2 ins (A)

delay_ms_loop
        goto    delayms#v(13)             ; 2 ins (C)
delayms#v(13)   goto    delayms#v(14)             ; 2 ins (C)
delayms#v(14)  goto    delayms#v(15)             ; 2 ins (C)
delayms#v(15)
delay_ms_entry_B
        goto    delayms#v(16)             ; 2 ins (B,C)
delayms#v(16)    goto    delayms#v(17)             ; 2 ins (B,C)
delayms#v(17)
delay_ms_entry_A
        goto    delayms#v(18)             ; 2 ins (A,B,C)
delayms#v(18)   goto    delayms#v(19)             ; 2 ins (A,B,C)
delayms#v(19)  goto    delayms#v(20)             ; 2 ins (A,B,C)
delayms#v(20)  addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfsc   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        return                  ; 2 ins (A)

        nop                     ; 1 ins (B)
        movlw   .250             ; 1 ins (B)
        goto    delay_ms_entry_B; 2 ins (B)


        ENDIF   ; CLOCK==20000000




;;--------------------------------------------------------------------------------

        IF      CLOCK==24000000

;; Timing analysis (at 24 MHz clock)
;; Load and call       =  3                     =     3
;; Static overhead = A = 13 + 8 * 248           =  1997
;; Outer loop      = B = 16 * (delay-1)         = -  16 +   16 * delay
;; Inner loop      = C =  8 * 248 * (delay-1)   = -1984 + 1984 * delay
;; Total           = 2000 * delay

delay_ms
        movwf   delay_wrk1
        movlw   3
        movwf   delay_wrk2
delay_macms1 movf    delay_wrk1,w
        movwf   delay_temp      ; 1 ins (A)
        movlw   .249             ; 1 ins (A)

delay_ms_loop
        goto    delayms#v(4)             ; 2 ins (A,B,C)
delayms#v(4)   goto    delayms#v(5)             ; 2 ins (A,B,C)
delayms#v(5)  addlw   -1              ; 1 ins (A,B,C)
        btfss   STATUS,Z        ; 1 ins (C) / 2 ins (A,B)
        goto    delay_ms_loop   ; 2 ins (C)

        decf    delay_temp,F    ; 1 ins (A,B)
        btfss   STATUS,Z        ; 2 ins (B) / 1 ins (A)
        goto    delaymc#v(7)
        decfsz  delay_wrk2,f
        goto    delay_macms1
        return                  ; 2 ins (A)

delaymc#v(7)    goto    delayms#v(6)             ; 2 ins (B)
delayms#v(6)    nop                     ; 1 ins (B)
        movlw   .249             ; 1 ins (B)
        goto    delay_ms_loop   ; 2 ins (B)


        ENDIF   ; CLOCK==24000000

;;--------------------------------------------------------------------------------


              IF      CLOCK==48000000          ; 12000000 ciclos por segundo (1 ciclo=83,333nS)
; ROTINA ESPCIFICA PARA FAMILIA 18F
; DEFINIR REGISTRADORES: 
;        delay_temp
;        delay_wrk1
;        delay_wrk2
;
; ROTINA GERADA POR:
; http://www.piclist.com/cgi-bin/delay.exe?Delay=0.001&Type=seconds&Regs=d1+d2+d3+d4&clock=48&name=Delay&CPU=PIC
;
delay_ms      movwf   delay_temp
delay_ms0     movlw   0x5F
              movwf   delay_wrk1
              movlw   0x0A
              movwf   delay_wrk2
delay_ms1     decfsz  delay_wrk1, f
              goto    delay_ms2
              decfsz  delay_wrk2, f
delay_ms2     goto    delay_ms1
              decfsz  delay_temp,f
              goto    delay_ms0
              return
              ENDIF
