; ------------------------------------------------------------------------------------------------------------
; LER MEMORIA EEPROM 93C66                                                                          02/08/2013                                              
;
; Leitura de uma EEPROM gravada em modo 16 bits.
; Uma chave ligada ao PORTA,0 permite ler cada palavra de 16 bits gravada na memoria.
; O PORTA,0 deve ter um resistor pull-up de 1K e a chave deve ser ligada ao PORTA,0 e ao GND.
; 
; ------------------------------------------------------------------------------------------------------------
        list      p=16F628A
        #include <p16F628A.inc>
        errorlevel  -302
; ----------------------------------------------------------------------------------
        __CONFIG   _CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT 
; ----------------------------------------------------------------------------------
             #include     "INFRA\CMCON_off.mac"
             #include     "INFRA\SR_4MVC.mac"
             #include     "INFRA\LCD_LINES.mac"
             #include     "INFRA\SET_CURSOR.mac"
             #include     "INFRA\hex2ascii.mac"
             #include     "INFRA\FAMILIAS.mac"
; ----------------------------------------------------------------------------------
             FAMILIAS
;
             UDATA  0x20
SR_1         RES          1             ; registrador auxiliar 1
SR_2         RES          1             ; registrador auxiliar 2
SR_3         RES          1             ; registrador auxiliar 3
SR_4         RES          1             ; registrador auxiliar 4
SR_5         RES          1             ; registrador auxiliar 5
LCD_COMMAND  RES          1             ; buffer de comando e dados
SR_TIPO      RES          1             ; tipo de LCD
SR_CLK       equ          7             ; em TRISx este bit deve ser saida
SR_DAT       equ          6             ; em TRISx este bit deve ser saida
SR_CHARS     equ          0             ; 0 - caracteres 5x7    1 - caracteres 5x10
#define      SR_TRIS      TRISB         ; I/O
#define      SR_DATA      PORTB,SR_DAT  ; DATA
#define      SR_CLOCK     PORTB,SR_CLK  ; CLOCK
;
delay_temp   RES    1                  ; registrador auxiliar para uso interno do LCD
;
EP_ADDRESS   RES    1                  ; endereco corrente na EPROM
EP_ICACHE    RES    4                  ; cache de instrucao da EPROM
EP_ILC       RES    1                  ; tamanho da instrucao
EP_DATA      RES    2                  ; 8 bits de dados
EP_TMP1      RES    1
EP_TMP2      RES    1
EP_TMP3      RES    1
; ----------------------------------------------------------------------------------
             #include     "INFRA\GPR.inc"
; ----------------------------------------------------------------------------------
#define      EP_TRIS      TRISB        ; porta da EPROM
#define      EP_PORT      PORTB        ; porta da EPROM
EP_CS        equ          3            ; pino EPROM Chip Select
EP_SK        equ          2            ; pino EPROM Clock
EP_DI        equ          1            ; pino EPROM Input
EP_DO        equ          4            ; pino EPROM Output
EP_STABLE    equ          .200         ; tempo para estabilizar a EEPROM no Chip Sel; STATUS - FUNCIONAect (micro segundos)
#define      EP_LED       PORTA
EP_LEDBIT    equ          2
; ----------------------------------------------------------------------------------
CLOCK        equ          4000000       ; frequencia de clock
; ----------------------------------------------------------------------------------
RESET_VECTOR CODE    0x000
             goto    inicio
INT_VECTOR   CODE    0x004
             retfie
; ----------------------------------------------------------------------------------
MAIN         CODE
msg000  SR_4MVC   ms000
ms000   addwf     PCL,f
        DT        "                ",0
;
msg001  SR_4MVC   ms001
ms001   addwf     PCL,f
        DT        "LER 93C66",0
;
msg004  SR_4MVC   ms004
ms004   addwf     PCL,f
        DT        "L",0
;
msg005  SR_4MVC   ms005
ms005   addwf     PCL,f
        DT        " 0x",0
;
msg006  SR_4MVC   ms006
ms006   addwf     PCL,f
        DT        " FIM",0
;
             LCD_LINES  0216
;
             HEX2ASCII
; ----------------------------------------------------------------------------------
inicio       nop
             CMCON_off
;
;            INICIALIZA AS PORTAS DE I/O
             clrf   PORTB
             bsf    STATUS,RP0

             movlw  b'00000001'
             movwf  PORTA


             movlw  b'00010000'
;                        |||||
;                        ||||+-------------------> nao usado
;                        |||+--------------------> DI - EPROM Input pin
;                        ||+---------------------> SK - EPROM Clock pin
;                        |+----------------------> CS - EPROM Chip Select pin
;                        +-----------------------> DO - EPROM Output pin 
             movwf  TRISB
             bcf    STATUS,RP0
             clrf   PORTA
             clrf   PORTB
; ----------------------------------------------------------------------------------
             call   SR_4BITS
             call   SR_LCDRESET
             call   msg001
; ----------------------------------------------------------------------------------
ler          clrf   EP_ADDRESS              ; aponta para o inicio da eprom
leitura      movlw  0x00                    ; 
             movwf  EP_DATA                 ; limpa o registrador de leitura
             call   EP_READ                 ; le a eprom

             call   set20                   ; limpa e posiciona LCD no inicio

             call   msg004                  ; LCD --> "L" 
             call   msg005                  ; LCD --> " 0x"
             movf   EP_ADDRESS,w            ; endereco atual da eprom
             call   HEXAG                   ; traduz endereco para ASCII
             call   hexShow                 ; mostra endereco no LCD --> L 0xNN

             call   msg005                  ; LCD --> " 0x"
             movf   EP_DATA+0,w             ; pega os 8 bits (MSB) do dado lido na eprom
             call   HEXAG                   ; traduz para ASCII
             call   hexShow                 ; vai mostrar MSB traduzido no LCD

             movf   EP_DATA+1,w             ; pega os 8 bits (LSB) do dado lido na eprom
             call   HEXAG                   ; traduz para ASCII
             call   hexShow                 ; vai mostrar LSB traduzido no LCD


f1           movf   PORTA,w                 ; le o PORTA
             movwf  EP_TMP1                 ; salva leitura temporariamente
             movlw  .80                     ; 80mS
             call   delay_ms                ; debounce da chave no PORTA 
             btfsc  EP_TMP1,0               ; se a chave nao foi pressionada,
             goto   f1                      ; vai esperar alguem pressionar a chave, senao...
;
             incfsz EP_ADDRESS,f            ; avanca o endereco na eprom (proximo byte)
             goto   leitura                 ; se o enderece for valido, vai ler a eprom, senao...
;
carryON      call   msg006                  ; mostra mensagem " FIM" no LCD
f2           movf   PORTA,w                 ; le o PORTA
             movwf  EP_TMP1                 ; salva leitura temporariamente
             movlw  .80                     ; 80mS
             call   delay_ms                ; debounce da chave no PORTA 
             btfsc  EP_TMP1,0               ; se a chave nao foi pressionada,
             goto   f2                      ; vai esperar alguem pressionar a chave, senao...
             goto   ler                     ; comeca a ler do inicio da eprom
; ----------------------------------------------------------------------------------
set20        SET_CURSOR 2,0,"i"
             call   msg000
             SET_CURSOR 2,0,"i"
             return
;
espera       movlw  .250
             call   delay_ms
             movlw  .250
             call   delay_ms
             movlw  .250
             call   delay_ms
             movlw  .250
             call   delay_ms
             return

hexShow      movf   R0,w
             movwf  EP_TMP3
             movf   R1,w
             movwf  LCD_COMMAND
             call   SR_SENDCHR
             movf   EP_TMP3,w
             movwf  LCD_COMMAND
             call   SR_SENDCHR
             return
; ----------------------------------------------------------------------------------
             #include  "INFRA\lcdsr44780.asm"
             #include  "INFRA\hexag.asm"
             #include  "INFRA\delay_ms.inc"
             #include  "INFRA\delay_us.inc"
             #include  "INFRA\eeprom_93c66.asm"
; ----------------------------------------------------------------------------------
             END